package com.kakao.sdk.partner.sample

import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.client.FriendPickerClient
import com.kakao.sdk.partner.friend.model.*
import com.kakao.sdk.partner.sample.databinding.ActivityFriendPickerSettingBinding

class FriendPickerSettingActivity : AppCompatActivity() {
    private lateinit var binding: ActivityFriendPickerSettingBinding
    private var pickerTitle: String? = null
    private var pickerFriendOrder: PickerFriendOrder? = null
    private var friendType: PickerFriendType? = null
    private var friendFilter: PickerFriendFilter? = null
    private var viewAppearance: ViewAppearance? = null
    private var countryCodes: List<String>? = null
    private var osFilter: PickerUsingOsFilter? = null
    private var orientation: PickerOrientation? = null
    private var placeHolder: String? = null
    private var disableSelectReason: DisableSelectReason? = null
    private var maxPickableCount: Int = 30
    private var minPickableCount: Int = 1

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityFriendPickerSettingBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val orderTypeArray = resources.getStringArray(R.array.friend_order)
        val friendTypeArray = resources.getStringArray(R.array.friend_type)
        val friendFilterArray = resources.getStringArray(R.array.friend_filter)
        val deviceOsArray = resources.getStringArray(R.array.device_os)
        val orientationArray = resources.getStringArray(R.array.picker_orientation)
        val viewAppearanceArray = resources.getStringArray(R.array.view_appearance)
        val disableSelectionReasonArray = resources.getStringArray(R.array.disable_selection_reason)

        binding.apply {
            cancelBtn.setOnClickListener {
                finish()
            }

            createBtn.setOnClickListener {
                pickerTitle = getPickerTitle()
                pickerFriendOrder = getFriendOrder(orderTypeArray)
                maxPickableCount = getMaxPickableCount()
                minPickableCount = getMinPickableCount()
                countryCodes = getCountryCodes()
                placeHolder = getSearchBarPlaceHolder()
                friendType = getPickerFriendType(friendTypeArray)
                friendFilter = getPickerFriendFilter(friendFilterArray)
                osFilter = getOsFilter(deviceOsArray)
                viewAppearance = getViewAppearance(viewAppearanceArray)
                orientation = getPickerOrientation(orientationArray)
                disableSelectReason = getDisableSelectionReason(disableSelectionReasonArray)

                val callback = { selectedUsers: SelectedUsers?, error: Throwable? ->
                    if (error != null) {
                        SdkLog.e("친구 정보 불러오기 실패 $error")
                        Toast.makeText(
                            this@FriendPickerSettingActivity,
                            "친구 정보 불러오기 실패 $error",
                            Toast.LENGTH_LONG
                        ).show()
                    } else {
                        SdkLog.i("친구 정보 불러오기 성공 $selectedUsers")
                        Toast.makeText(
                            this@FriendPickerSettingActivity,
                            "친구 정보 불러오기 성공 $selectedUsers",
                            Toast.LENGTH_LONG
                        ).show()
                    }
                }

                val param = getPickerRequestParams()
                if (popupCb.isChecked) {
                    if (singleCb.isChecked) {
                        FriendPickerClient.instance.selectSinglePopup(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    } else {
                        FriendPickerClient.instance.selectMultiplePopup(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    }
                } else {
                    if (singleCb.isChecked) {
                        FriendPickerClient.instance.selectSingle(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    } else {
                        FriendPickerClient.instance.selectMultiple(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    }
                }
            }
        }
    }

    private fun ActivityFriendPickerSettingBinding.getDisableSelectionReason(
        disableSelectionReasonArray: Array<String>
    ) = when (msgBlockedSpinner.selectedItem) {
        disableSelectionReasonArray[1] -> DisableSelectReason.MSG_BLOCKED
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getPickerOrientation(
        orientationArray: Array<String>
    ) = when (orientationSpinner.selectedItem) {
        orientationArray[1] -> PickerOrientation.AUTO
        orientationArray[2] -> PickerOrientation.PORTRAIT
        orientationArray[3] -> PickerOrientation.LANDSCAPE
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getViewAppearance(
        viewAppearanceArray: Array<String>
    ) = when (themeSpinner.selectedItem) {
        viewAppearanceArray[1] -> ViewAppearance.AUTO
        viewAppearanceArray[2] -> ViewAppearance.LIGHT
        viewAppearanceArray[3] -> ViewAppearance.DARK
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getOsFilter(
        deviceOsArray: Array<String>
    ) = when (osSpinner.selectedItem) {
        deviceOsArray[1] -> PickerUsingOsFilter.ALL
        deviceOsArray[2] -> PickerUsingOsFilter.ANDROID
        deviceOsArray[3] -> PickerUsingOsFilter.IOS
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getPickerFriendFilter(
        friendFilterArray: Array<String>
    ) = when (friendFilterSpinner.selectedItem) {
        friendFilterArray[1] -> PickerFriendFilter.REGISTERED
        friendFilterArray[2] -> PickerFriendFilter.INVITABLE
        friendFilterArray[3] -> PickerFriendFilter.NONE
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getPickerFriendType(
        friendTypeArray: Array<String>
    ) = when (friendTypeSpinner.selectedItem) {
        friendTypeArray[1] -> PickerFriendType.TALK
        friendTypeArray[2] -> PickerFriendType.STORY
        friendTypeArray[3] -> PickerFriendType.TALKSTORY
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getFriendOrder(
        orderTypeArray: Array<String>
    ) = when (friendOrderSpinner.selectedItem) {
        orderTypeArray[1] -> PickerFriendOrder.AGE
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getSearchBarPlaceHolder() =
        if (placeholderEt.text.toString().isEmpty()) {
            null
        } else {
            placeholderEt.text.toString()
        }

    private fun ActivityFriendPickerSettingBinding.getCountryCodes() =
        if (countryCodeEt.text.isNullOrBlank()) {
            null
        } else {
            countryCodeEt.text.toString().split(',').toList()
        }

    private fun ActivityFriendPickerSettingBinding.getMinPickableCount() =
        if (minCountEt.text.toString().isBlank()) {
            minCountEt.hint.toString().toInt()
        } else {
            minCountEt.text.toString().toInt()
        }

    private fun ActivityFriendPickerSettingBinding.getMaxPickableCount() =
        if (maxCountEt.text.toString().isBlank()) {
            maxCountEt.hint.toString().toInt()
        } else {
            maxCountEt.text.toString().toInt()
        }

    private fun ActivityFriendPickerSettingBinding.getPickerTitle() =
        if (titleEt.text.toString().isEmpty()) {
            null
        } else {
            titleEt.text.toString()
        }

    private fun getPickerRequestParams(): PickerRequestParams {
        return binding.run {
            PickerRequestParams(
                pickerTitle,
                pickerFriendOrder,
                friendType,
                friendFilter,
                viewAppearance,
                countryCodes,
                osFilter,
                orientation,
                searchbarCb.isChecked,
                placeHolder,
                indexCb.isChecked,
                myProfileCb.isChecked,
                myFavoriteCb.isChecked,
                disableSelectReason,
                maxPickableCount,
                minPickableCount,
                pickedCb.isChecked,
                allProfileCb.isChecked
            )
        }
    }

    companion object {
        const val TAG = "KakaoSDKSample"
    }
}